__author__ = "Simon Nilsson"

import os
import threading
from tkinter import *
from typing import Union

from simba.mixins.config_reader import ConfigReader
from simba.mixins.feature_extraction_mixin import FeatureExtractionMixin
from simba.mixins.pop_up_mixin import PopUpMixin
from simba.plotting.ROI_feature_visualizer import ROIfeatureVisualizer
from simba.plotting.ROI_feature_visualizer_mp import \
    ROIfeatureVisualizerMultiprocess
from simba.ui.tkinter_functions import (CreateLabelFrameWithIcon, Entry_Box,
                                        SimbaButton, SimbaCheckbox,
                                        SimBADropDown, SimBALabel)
from simba.utils.checks import check_float, check_nvidea_gpu_available
from simba.utils.enums import Formats, Links
from simba.utils.errors import NoFilesFoundError, NoROIDataError
from simba.utils.read_write import find_all_videos_in_directory, str_2_bool

ROI_CENTERS = "roi_centers"
ROI_EAR_TAGS = "roi_ear_tags"
DIRECTIONALITY = "directionality"
DIRECTIONALITY_STYLE = "directionality_style"
BORDER_COLOR = "border_color"
POSE = "pose_estimation"
ANIMAL_NAMES = "animal_names"


class VisualizeROIFeaturesPopUp(PopUpMixin, ConfigReader, FeatureExtractionMixin):

    """
    :example:
    >>> _ = VisualizeROIFeaturesPopUp(config_path='/Users/simon/Desktop/envs/simba/troubleshooting/mouse_open_field/project_folder/project_config.ini')
    """
    def __init__(self, config_path: Union[str, os.PathLike]):

        ConfigReader.__init__(self, config_path=config_path, read_video_info=False)
        self.video_file_paths = find_all_videos_in_directory(directory=self.video_dir, as_dict=True)
        self.video_list = [k for k in self.video_file_paths.keys()]
        if len(self.video_list) == 0:
            raise NoFilesFoundError(msg=f"No videos in SimBA project {self.video_dir} directory. Import videos into you SimBA project to visualize ROI features.",source=self.__class__.__name__,)
        if not os.path.isfile(self.roi_coordinates_path):
            raise NoFilesFoundError(msg=f"No ROI data found in SimBA project (expected at path {self.roi_coordinates_path}). Draw ROIs before visualize ROI features.", source=self.__class__.__name__, )
        self.read_roi_data()
        self.video_names_w_rois = list(self.video_names_w_rois)
        self.video_file_paths = {k:v for k, v in self.video_file_paths.items() if k in self.video_names_w_rois}
        self.video_list = [k for k in self.video_file_paths.keys()]
        if len(self.video_list) == 0:
            raise NoROIDataError(msg=f'None of the imported videos in the project has ROI data associated with them. Draw ROIs on the videos in the project before visualize ROI features')
        self.max_video_name_len = len(max(self.video_list, key=len))
        self.directing_viable = NORMAL if self.check_directionality_viable()[0] else DISABLED
        self.gpu_available = NORMAL if check_nvidea_gpu_available() else DISABLED
        PopUpMixin.__init__(self, title="VISUALIZE ROI FEATURES", icon='shapes_small')
        FeatureExtractionMixin.__init__(self, config_path=self.config_path)
        self.settings_frm = CreateLabelFrameWithIcon(parent=self.main_frm, header="SETTINGS", icon_name='settings', icon_link=Links.ROI_FEATURES_PLOT.value)
        self.threshold_entry_box = Entry_Box(self.settings_frm, "PROBABILITY THRESHOLD:", "25", value=0.0, entry_box_width=15)
        threshold_label = SimBALabel(parent=self.settings_frm, txt="NOTE: body-part locations detected with probabilities \n below this threshold are filtered.", font=Formats.FONT_REGULAR_ITALICS.value)

        self.border_clr_dropdown = SimBADropDown(parent=self.settings_frm, dropdown_options=list(self.colors_dict.keys()), label='BORDER COLOR:', label_width=25, dropdown_width=15, value='Black')
        self.show_directionality_dropdown = SimBADropDown(parent=self.settings_frm, dropdown_options=['FALSE', 'LINES', 'FUNNEL'], label='SHOW DIRECTIONALITY:', label_width=25, dropdown_width=15, value='FALSE', state=self.directing_viable)
        self.cpu_cores_dropdown = SimBADropDown(parent=self.settings_frm, dropdown_options=list(range(1, self.cpu_cnt+1)), label='CPU CORES:', label_width=25, dropdown_width=15, value=1)
        self.gpu_dropdown = SimBADropDown(parent=self.settings_frm, dropdown_options=['FALSE', 'TRUE'], label='USE GPU:', label_width=25, dropdown_width=15, value='FALSE', state=self.gpu_available)
        show_pose_cb, self.show_pose_var = SimbaCheckbox(parent=self.settings_frm, txt='SHOW POSE', txt_img='pose', val=True)
        show_roi_center_cb, self.show_ROI_centers_var = SimbaCheckbox(parent=self.settings_frm, txt='SHOW ROI CENTERS', txt_img='center', val=True)
        show_roi_tags_cb, self.show_ROI_tags_var = SimbaCheckbox(parent=self.settings_frm, txt='SHOW ROI EAR TAGS', txt_img='ear_small', val=True)
        show_animal_names_cb, self.show_animal_names_var = SimbaCheckbox(parent=self.settings_frm, txt='SHOW ANIMAL NAMES', txt_img='label_yellow', val=False)

        self.body_parts_frm = CreateLabelFrameWithIcon(parent=self.main_frm, header="SELECT BODY-PARTS", icon_name='pose', icon_link=Links.ROI_FEATURES_PLOT.value)
        self.animal_cnt_dropdown = SimBADropDown(parent=self.body_parts_frm, dropdown_options=list(range(1, self.animal_cnt + 1)), label='NUMBER OF ANIMALS:', label_width=25, dropdown_width=15, value=1, command=self.__populate_bp_dropdown)
        self.__populate_bp_dropdown(bp_cnt=1)
        self.animal_cnt_dropdown.grid(row=0, column=0, sticky=NW)

        self.single_video_frm = CreateLabelFrameWithIcon(parent=self.main_frm, header="VISUALIZE SINGLE VIDEO", icon_name='video', icon_link=Links.ROI_FEATURES_PLOT.value)
        self.single_video_dropdown = SimBADropDown(parent=self.single_video_frm, dropdown_options=self.video_list, label='SELECT VIDEO:', label_width=25, dropdown_width=self.max_video_name_len+10, value=self.video_list[0])
        self.single_video_btn = SimbaButton(parent=self.single_video_frm, txt="VISUALIZE ROI FEATURES: SINGLE VIDEO", img='rocket', font=Formats.FONT_REGULAR.value, cmd=self.run, cmd_kwargs={'multiple': False}, width=240, txt_clr='blue')


        self.all_videos_frm = CreateLabelFrameWithIcon(parent=self.main_frm, header="VISUALIZE ALL VIDEOS", icon_name='video', icon_link=Links.ROI_FEATURES_PLOT.value)
        self.all_videos_btn = SimbaButton(parent=self.all_videos_frm, txt="VISUALIZE ROI FEATURES: ALL VIDEOS", img='rocket', font=Formats.FONT_REGULAR.value, cmd=self.run, cmd_kwargs={'multiple': True}, width=240, txt_clr='red')

        self.settings_frm.grid(row=0, column=0, sticky=NW)
        self.threshold_entry_box.grid(row=0, column=0, sticky=NW)
        threshold_label.grid(row=1, column=0, sticky=NW)
        self.cpu_cores_dropdown.grid(row=2, column=0, sticky=NW)
        self.border_clr_dropdown.grid(row=3, column=0, sticky=NW)
        self.show_directionality_dropdown.grid(row=4, column=0, sticky=NW)
        self.gpu_dropdown.grid(row=5, column=0, sticky=NW)
        show_pose_cb.grid(row=6, column=0, sticky=NW)
        show_roi_center_cb.grid(row=7, column=0, sticky=NW)
        show_roi_tags_cb.grid(row=8, column=0, sticky=NW)
        show_animal_names_cb.grid(row=9, column=0, sticky=NW)
        self.body_parts_frm.grid(row=1, column=0, sticky=NW)
        self.single_video_frm.grid(row=2, column=0, sticky=NW)
        self.single_video_dropdown.grid(row=0, column=0, sticky=NW)
        self.single_video_btn.grid(row=1, column=0, sticky=NW)
        self.all_videos_frm.grid(row=3, column=0, sticky=NW)
        self.all_videos_btn.grid(row=0, column=0, sticky=NW)
        self.main_frm.mainloop()

    def __populate_bp_dropdown(self, bp_cnt: int):
        if hasattr(self, "bp_dropdown_dict"):
            if len(self.bp_dropdown_dict.keys()) != bp_cnt:
                for k, v in self.bp_dropdown_dict.items():
                    v.destroy()

        self.bp_dropdown_dict = {}
        for cnt in range(int(self.animal_cnt_dropdown.getChoices())):
            self.bp_dropdown_dict[cnt] = SimBADropDown(parent=self.body_parts_frm, dropdown_options=self.body_parts_lst, label=self.multi_animal_id_list[cnt], label_width=25, dropdown_width=15, value=self.body_parts_lst[cnt])
            self.bp_dropdown_dict[cnt].grid(row=cnt + 1, column=0, sticky=NW)

    def run(self, multiple: bool):
        check_float(name="Body-part probability threshold", value=self.threshold_entry_box.entry_get, min_value=0.0, max_value=1.0)
        show_direction = True if self.show_directionality_dropdown.get_value() in ['FUNNEL', 'LINES'] else False
        direction_style = self.show_directionality_dropdown.get_value()
        core_cnt = int(self.cpu_cores_dropdown.get_value())
        gpu = str_2_bool(self.gpu_dropdown.get_value())
        style_attr = {ROI_CENTERS: self.show_ROI_centers_var.get(),
                      ROI_EAR_TAGS: self.show_ROI_tags_var.get(),
                      POSE: self.show_pose_var.get(),
                      ANIMAL_NAMES: self.show_animal_names_var.get(),
                      DIRECTIONALITY: show_direction,
                      BORDER_COLOR: self.colors_dict[self.border_clr_dropdown.getChoices()],
                      DIRECTIONALITY_STYLE: direction_style.lower()}

        if multiple:
            video_paths = [v for k, v in self.video_file_paths.items()]
        else:
            video_paths = [self.video_file_paths[self.single_video_dropdown.getChoices()]]
        body_parts = [v.getChoices() for v in self.bp_dropdown_dict.values()]
        for video_path in video_paths:
            if self.cpu_cores_dropdown.get_value() == 1:
                roi_feature_visualizer = ROIfeatureVisualizer(config_path=self.config_path,
                                                              video_path=video_path,
                                                              body_parts=body_parts,
                                                              style_attr=style_attr)
            else:
                roi_feature_visualizer = ROIfeatureVisualizerMultiprocess(config_path=self.config_path,
                                                                          video_path=video_path,
                                                                          body_parts=body_parts,
                                                                          style_attr=style_attr,
                                                                          core_cnt=core_cnt,
                                                                          gpu=gpu)
            threading.Thread(target=roi_feature_visualizer.run()).start()



#_ = VisualizeROIFeaturesPopUp(config_path=r"C:\troubleshooting\RAT_NOR\project_folder\project_config.ini")


#_ = VisualizeROIFeaturesPopUp(config_path=r"C:\troubleshooting\two_black_animals_14bp\project_folder\project_config.ini")


#_ = VisualizeROIFeaturesPopUp(config_path=r"C:\troubleshooting\roi_duplicates\project_folder\project_config.ini")


# _ = VisualizeROIFeaturesPopUp(config_path='/Users/simon/Desktop/envs/troubleshooting/Two_animals_16bps/project_folder/project_config.ini')
# _ = VisualizeROIFeaturesPopUp(config_path='/Users/simon/Desktop/envs/troubleshooting/Termites_5/project_folder/project_config.ini')
# ROIAnalysisPopUp(config_path='/Users/simon/Desktop/envs/troubleshooting/Termites_5/project_folder/project_config.ini')

# _ = VisualizeROIFeaturesPopUp(config_path='/Users/simon/Desktop/envs/troubleshooting/two_black_animals_14bp/project_folder/project_config.ini')



__author__ = "Simon Nilsson"

import os
from tkinter import *
from typing import Union

from simba.mixins.config_reader import ConfigReader
from simba.mixins.pop_up_mixin import PopUpMixin
from simba.plotting.gantt_creator import GanttCreatorSingleProcess
from simba.plotting.gantt_creator_mp import GanttCreatorMultiprocess
from simba.ui.tkinter_functions import (CreateLabelFrameWithIcon, DropDownMenu,
                                        SimbaButton, SimbaCheckbox,
                                        SimBADropDown)
from simba.utils.checks import check_if_filepath_list_is_empty, check_int
from simba.utils.enums import Formats, Keys, Links, Options, Paths
from simba.utils.errors import NoSpecifiedOutputError
from simba.utils.read_write import find_files_of_filetypes_in_directory


class GanttPlotPopUp(PopUpMixin, ConfigReader):

    """
    :example:
    >>>  _ = GanttPlotPopUp(config_path=r"C:\troubleshooting\RAT_NOR\project_folder\project_config.ini")
    """
    def __init__(self, config_path: Union[str, os.PathLike]):
        ConfigReader.__init__(self, config_path=config_path, read_video_info=False)
        check_if_filepath_list_is_empty(filepaths=self.machine_results_paths,error_msg=f"SIMBA ERROR: Zero files found in the {self.machine_results_dir} directory. Create classification results before visualizing gantt charts",)
        palettes = Options.PALETTE_OPTIONS_CATEGORICAL.value + Options.PALETTE_OPTIONS.value
        self.data_paths = find_files_of_filetypes_in_directory(directory=self.machine_results_dir, extensions=[f'.{self.file_type}'], as_dict=True)
        max_file_name_len = max(len(k) for k in self.data_paths) + 5

        PopUpMixin.__init__(self, config_path=config_path, title="VISUALIZE GANTT PLOTS", icon='gantt_small')
        self.style_settings_frm = CreateLabelFrameWithIcon(parent=self.main_frm, header="STYLE SETTINGS", icon_name='settings', icon_link=Links.GANTT_PLOTS.value, relief='solid', padx=5, pady=5)
        self.resolution_dropdown = SimBADropDown(parent=self.style_settings_frm, dropdown_options=self.resolutions, label='GANTT PLOT RESOLUTION: ', label_width=30, dropdown_width=30, value='640×480')
        self.font_size_dropdown = SimBADropDown(parent=self.style_settings_frm, dropdown_options=list(range(1, 26)), label='TEXT SIZE: ', label_width=30, dropdown_width=30, value=8)
        self.font_rotation_dropdown = SimBADropDown(parent=self.style_settings_frm, dropdown_options=list(range(0, 182, 2)), label='TEXT ROTATION (°): ', label_width=30, dropdown_width=30, value=0)
        self.palette_dropdown = SimBADropDown(parent=self.style_settings_frm, dropdown_options=palettes, label='COLOR PALETTE: ', label_width=30, dropdown_width=30, value='Set1')
        self.core_dropdown = SimBADropDown(parent=self.style_settings_frm, dropdown_options=list(range(1, self.cpu_cnt+1)), label='CPU CORES: ', label_width=30, dropdown_width=30, value=int(self.cpu_cnt/2))

        self.settings_frm = CreateLabelFrameWithIcon(parent=self.main_frm, header="VISUALIZATION SETTINGS", icon_name='eye', icon_link=Links.GANTT_PLOTS.value, relief='solid', padx=5, pady=5)
        gantt_frames_cb, self.gantt_frames_var = SimbaCheckbox(parent=self.settings_frm, txt='CREATE FRAMES', txt_img='frames', val=False)
        gantt_videos_cb, self.gantt_videos_var = SimbaCheckbox(parent=self.settings_frm, txt='CREATE VIDEOS', txt_img='video', val=False)
        gantt_last_frame_cb, self.gantt_last_frame_var = SimbaCheckbox(parent=self.settings_frm, txt='CREATE LAST FRAME', txt_img='finish', val=True)

        self.run_single_video_frm= CreateLabelFrameWithIcon(parent=self.main_frm, header="SINGLE VIDEO", icon_name='eye', icon_link=Links.GANTT_PLOTS.value, relief='solid', padx=5, pady=5)
        self.run_single_video_btn = SimbaButton(parent=self.run_single_video_frm, txt="VIDEO", txt_clr="blue", img='rocket', font=Formats.FONT_REGULAR.value, cmd=self.__create_gantt_plots, cmd_kwargs={'multiple': False})
        self.single_video_dropdown = SimBADropDown(parent=self.run_single_video_frm, dropdown_options=list(self.data_paths.keys()), label='VIDEO', label_width=20, dropdown_width=max_file_name_len, value=list(self.data_paths.keys())[0])

        self.run_multiple_videos = CreateLabelFrameWithIcon(parent=self.main_frm, header="MULTIPLE VIDEO(S)", icon_name='stack', icon_link=Links.GANTT_PLOTS.value, relief='solid', padx=5, pady=5)
        self.run_multiple_video_btn = SimbaButton(parent=self.run_multiple_videos, txt=f"Create multiple videos ({len(list(self.data_paths.keys()))} video(s) found)", txt_clr="blue", img='rocket', font=Formats.FONT_REGULAR.value, cmd=self.__create_gantt_plots, cmd_kwargs={'multiple': True})
        self.style_settings_frm.grid(row=0, sticky=NW, padx=10, pady=10)
        self.resolution_dropdown.grid(row=0, sticky=NW)
        self.font_size_dropdown.grid(row=1, sticky=NW)
        self.font_rotation_dropdown.grid(row=2, sticky=NW)
        self.palette_dropdown.grid(row=3, sticky=NW)
        self.core_dropdown.grid(row=4, sticky=NW)

        self.settings_frm.grid(row=1, sticky=NW, padx=10, pady=10)
        gantt_videos_cb.grid(row=0, sticky=NW)
        gantt_frames_cb.grid(row=1, sticky=W)
        gantt_last_frame_cb.grid(row=2, sticky=NW)

        self.run_single_video_frm.grid(row=2, sticky=NW)
        self.run_single_video_btn.grid(row=0, sticky=NW)
        self.single_video_dropdown.grid(row=0, column=1, sticky=NW)

        self.run_multiple_videos.grid(row=3, sticky=NW)
        self.run_multiple_video_btn.grid(row=0, sticky=NW)

        self.main_frm.mainloop()

    def __create_gantt_plots(self,
                             multiple: bool):

        width = int(self.resolution_dropdown.getChoices().split("×")[0])
        height = int(self.resolution_dropdown.getChoices().split("×")[1])
        font_size = int(self.font_size_dropdown.get_value())
        font_rotation = int(self.font_rotation_dropdown.get_value())
        core_cnt = int(self.core_dropdown.get_value())
        frame_setting = self.gantt_frames_var.get()
        video_setting = self.gantt_videos_var.get()
        last_frm_setting = self.gantt_last_frame_var.get()
        palette = self.palette_dropdown.get_value()

        if not frame_setting and not video_setting and not last_frm_setting:
            raise NoSpecifiedOutputError(msg="SIMBA ERROR: Please select gantt videos, frames, and/or last frame.")

        if multiple:
            data_paths = list(self.data_paths.values())
        else:
            data_paths = [self.data_paths[self.single_video_dropdown.getChoices()]]

        if core_cnt > 1:
            gantt_creator = GanttCreatorMultiprocess(config_path=self.config_path,
                                                     frame_setting=frame_setting,
                                                     video_setting=video_setting,
                                                     last_frm_setting=last_frm_setting,
                                                     data_paths=data_paths,
                                                     width=width,
                                                     height=height,
                                                     font_size=font_size,
                                                     font_rotation=font_rotation,
                                                     core_cnt=core_cnt,
                                                     palette=palette)

        else:
            gantt_creator = GanttCreatorSingleProcess(config_path=self.config_path,
                                                      frame_setting=frame_setting,
                                                      video_setting=video_setting,
                                                      last_frm_setting=last_frm_setting,
                                                      data_paths=data_paths,
                                                      width=width,
                                                      height=height,
                                                      font_size=font_size,
                                                      font_rotation=font_rotation,
                                                      palette=palette)
        gantt_creator.run()



#_ = GanttPlotPopUp(config_path=r"C:\troubleshooting\mitra\project_folder\project_config.ini")

 # _ = GanttPlotPopUp(config_path=r"C:\troubleshooting\RAT_NOR\project_folder\project_config.ini")



from tkinter import *

import numpy as np

from simba.mixins.pop_up_mixin import PopUpMixin
from simba.third_party_label_appenders.transform.labelme_to_yolo import \
    LabelmeBoundingBoxes2YoloBoundingBoxes
from simba.ui.tkinter_functions import (CreateLabelFrameWithIcon, FolderSelect,
                                        SimBADropDown)
from simba.utils.checks import check_if_dir_exists
from simba.utils.read_write import (find_files_of_filetypes_in_directory,
                                    str_2_bool)

TRAIN_SIZE_OPTIONS = list(np.arange(10, 110, 10))

class LabelmeBbox2YoloBboxPopUp(PopUpMixin):

    """
    >>> LabelmeBbox2YoloBboxPopUp()
    """

    def __init__(self):

        PopUpMixin.__init__(self, title="LABELME BOUNDING BOXES -> YOLO BOUNDING BOXES ", icon='dlc_1')

        settings_frm = CreateLabelFrameWithIcon(parent=self.main_frm, header="SETTINGS", icon_name='settings')
        self.labelme_dir = FolderSelect(settings_frm, folderDescription="LABELME DIRECTORY:", lblwidth=35)
        self.save_dir = FolderSelect(settings_frm, folderDescription="SAVE DIRECTORY:", lblwidth=35)
        self.verbose_dropdown = SimBADropDown(parent=settings_frm, dropdown_options=['TRUE', 'FALSE'], label="VERBOSE:", label_width=35, dropdown_width=40, value='TRUE')
        self.obb_dropdown = SimBADropDown(parent=settings_frm, dropdown_options=['TRUE', 'FALSE'], label="OBB:", label_width=35, dropdown_width=40, value='TRUE')
        self.clahe_dropdown = SimBADropDown(parent=settings_frm, dropdown_options=['TRUE', 'FALSE'], label="CLAHE:", label_width=35, dropdown_width=40, value='FALSE')
        self.greyscale_dropdown = SimBADropDown(parent=settings_frm, dropdown_options=['TRUE', 'FALSE'], label="GREYSCALE:", label_width=35, dropdown_width=40, value='FALSE')
        self.train_size_dropdown = SimBADropDown(parent=settings_frm, dropdown_options=TRAIN_SIZE_OPTIONS, label="TRAIN SIZE (%): ", label_width=35, dropdown_width=40, value=70)


        settings_frm.grid(row=0, column=0, sticky=NW)
        self.labelme_dir .grid(row=0, column=0, sticky=NW)
        self.save_dir.grid(row=1, column=0, sticky=NW)
        self.obb_dropdown.grid(row=2, column=0, sticky=NW)
        self.clahe_dropdown.grid(row=3, column=0, sticky=NW)
        self.greyscale_dropdown.grid(row=4, column=0, sticky=NW)
        self.train_size_dropdown.grid(row=5, column=0, sticky=NW)
        self.verbose_dropdown.grid(row=6, column=0, sticky=NW)
        self.create_run_frm(run_function=self.run)
        self.main_frm.mainloop()

    def run(self):
        save_dir = self.save_dir.folder_path
        labelme_dir = self.labelme_dir.folder_path
        verbose = str_2_bool(self.verbose_dropdown.get_value())
        clahe = str_2_bool(self.clahe_dropdown.get_value())
        obb = str_2_bool(self.obb_dropdown.get_value())
        grey = str_2_bool(self.greyscale_dropdown.get_value())
        train_size = float(self.train_size_dropdown.get_value()) / 100

        check_if_dir_exists(in_dir=save_dir, source=f'{self.__class__.__name__} LABELME DIRECTORY')
        check_if_dir_exists(in_dir=labelme_dir, source=f'{self.__class__.__name__} LABELME SAVE DIR')

        _ = find_files_of_filetypes_in_directory(directory=labelme_dir, extensions=['.json'], raise_error=True)

        runner = LabelmeBoundingBoxes2YoloBoundingBoxes(labelme_dir=labelme_dir, save_dir=save_dir, obb=obb, verbose=verbose, clahe=clahe, train_size=train_size, greyscale=grey)
        runner.run()



#LabelmeBbox2YoloBboxPopUp()


import os.path
from tkinter import *

from simba.mixins.pop_up_mixin import PopUpMixin
from simba.ui.tkinter_functions import (CreateLabelFrameWithIcon, FolderSelect,
                                        SimbaButton, SimBADropDown)
from simba.utils.checks import check_if_dir_exists, check_nvidea_gpu_available
from simba.utils.read_write import (find_all_videos_in_directory,
                                    find_core_cnt, get_video_meta_data,
                                    str_2_bool)
from simba.video_processors.videos_to_frames import video_to_frames

ALL_IMAGES = 'ALL IMAGES'
batch_size_options = list(range(100, 5100, 100))
batch_size_options.insert(0, ALL_IMAGES)

class MultipleVideos2FramesPopUp(PopUpMixin):
    def __init__(self):
        PopUpMixin.__init__(self, title="EXTRACT FRAMES FROM SINGLE VIDEO", icon='frames')
        core_cnt = find_core_cnt()[0]
        settings_frm = CreateLabelFrameWithIcon(parent=self.main_frm, header='SETTINGS', icon_name='settings', padx=5, pady=5, relief='solid')

        self.video_dir = FolderSelect(parent=settings_frm, folderDescription="VIDEO DIRECTORY:", lblwidth=50)
        self.save_dir = FolderSelect(parent=settings_frm, folderDescription="SAVE DIRECTORY:", lblwidth=50)
        self.core_cnt_dropdown = SimBADropDown(parent=settings_frm, dropdown_options=list(range(1, core_cnt+1)), label='CORE COUNT: ', label_width=50, value=int(core_cnt/2))
        self.quality_dropdown = SimBADropDown(parent=settings_frm, dropdown_options=list(range(1, 101)), label='IMAGE QUALITY: ', label_width=50, value=90, state='disabled')
        self.img_format = SimBADropDown(parent=settings_frm, dropdown_options=['jpeg', 'png', 'webp'], label='IMAGE FORMAT: ', label_width=50, value='png', command=self._inactivate_quality)
        self.verbose_dropdown = SimBADropDown(parent=settings_frm, dropdown_options=['TRUE', 'FALSE'], label='VERBOSE: ', label_width=50, value='TRUE')
        self.greyscale_dropdown = SimBADropDown(parent=settings_frm, dropdown_options=['TRUE', 'FALSE'], label='GREYSCALE: ', label_width=50, value='FALSE')
        self.bw_dropdown = SimBADropDown(parent=settings_frm, dropdown_options=['TRUE', 'FALSE'], label='BLACK & WHITE: ',label_width=50, value='FALSE')
        self.clahe_dropdown = SimBADropDown(parent=settings_frm, dropdown_options=['TRUE', 'FALSE'], label='CLAHE: ', label_width=50, value='FALSE')
        self.include_fn_dropdown = SimBADropDown(parent=settings_frm, dropdown_options=['TRUE', 'FALSE'], label='INCLUDE VIDEO NAME IN IMAGE NAMES: ',label_width=50, value='FALSE')

        settings_frm.grid(row=0, column=0, sticky=NW, padx=10, pady=10)
        self.video_dir.grid(row=0, column=0, sticky=NW)
        self.save_dir.grid(row=1, column=0, sticky=NW)
        self.core_cnt_dropdown.grid(row=2, column=0, sticky=NW)
        self.quality_dropdown.grid(row=3, column=0, sticky=NW)
        self.img_format.grid(row=4, column=0, sticky=NW)
        self.greyscale_dropdown.grid(row=5, column=0, sticky=NW)
        self.bw_dropdown.grid(row=6, column=0, sticky=NW)
        self.clahe_dropdown.grid(row=7, column=0, sticky=NW)
        self.include_fn_dropdown.grid(row=8, column=0, sticky=NW)
        self.verbose_dropdown.grid(row=9, column=0, sticky=NW)

        run_frm = CreateLabelFrameWithIcon(parent=self.main_frm, header='RUN', icon_name='run', padx=5, pady=5, relief='solid')
        run_btn = SimbaButton(parent=run_frm, txt='RUN', img='rocket', cmd=self._run)

        run_frm.grid(row=1, column=0, sticky=NW, padx=10, pady=10)
        run_btn.grid(row=0, column=0, sticky=NW)
        self.main_frm.mainloop()

    def _inactivate_quality(self, img_format):
        if img_format != 'jpeg': self.quality_dropdown.disable()
        else: self.quality_dropdown.enable()

    def _run(self):
        video_dir = self.video_dir.folder_path
        video_dict = find_all_videos_in_directory(directory=video_dir, as_dict=True, raise_error=True)
        save_dir = self.save_dir.folder_path
        core_cnt = int(self.core_cnt_dropdown.get_value())
        quality = int(self.quality_dropdown.get_value())
        verbose = str_2_bool(self.verbose_dropdown.get_value())
        img_format = self.img_format.get_value()
        check_if_dir_exists(in_dir=save_dir)
        bw = str_2_bool(self.bw_dropdown.get_value())
        grey = str_2_bool(self.greyscale_dropdown.get_value())
        clahe = str_2_bool(self.clahe_dropdown.get_value())
        include_fn = str_2_bool(self.include_fn_dropdown.get_value())

        for video_cnt, (video_name, video_path) in enumerate(video_dict.items()):
            video_meta_data = get_video_meta_data(video_path=video_path)
            video_save_dir = os.path.join(save_dir, video_meta_data["video_name"])
            print(f'Extracting frames from {video_meta_data["video_name"]} and saving at {video_save_dir} (video {video_cnt+1})....')
            video_to_frames(video_path=video_path,
                            save_dir=save_dir,
                            core_cnt=core_cnt,
                            quality=quality,
                            img_format=img_format,
                            verbose=verbose,
                            clahe=clahe,
                            black_and_white=bw,
                            greyscale=grey,
                            include_video_name_in_filename=include_fn)

#MultipleVideos2FramesPopUp()



